"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const javaUtil_1 = require("../java/javaUtil");
const RefType_1 = __importDefault(require("./refs/RefType"));
const GenericRef_1 = __importDefault(require("./refs/GenericRef"));
const beanUtils_1 = require("../java/beanUtils");
const PropertyBuilder_1 = require("./PropertyBuilder");
class Parameter {
    constructor() {
        this.required = false;
        this.vendorExtensions = javaUtil_1.newHashMap();
    }
    isUniqueItems() {
        return this.uniqueItems;
    }
    isExclusiveMinimum() {
        return this.exclusiveMinimum != null;
    }
    isExclusiveMaximum() {
        return this.exclusiveMaximum != null;
    }
    setFormat(format) {
        this.format = format;
    }
    getFormat() {
        return this.format;
    }
    getIn() {
        return this.in;
    }
    isReadOnly() {
        return this.readOnly;
    }
    setVendorExtension(name, value) {
        if (name.startsWith('-x')) {
            this.vendorExtensions.put(name, value);
        }
    }
    getVendorExtensions() {
        return this.vendorExtensions;
    }
    getDefaultValue() {
        return this.defaultValue;
    }
    setDefaultValue(defaultValue) {
        this.defaultValue = defaultValue;
    }
    getDefault() {
        if (this.defaultValue == null || this.defaultValue.length == 0) {
            return null;
        }
        return this.defaultValue;
    }
    setDefault(defaultValue) {
        this.defaultValue = defaultValue == null ? null : defaultValue.toString();
    }
    copy() {
        return beanUtils_1.apply(new this.constructor(), this);
    }
}
exports.Parameter = Parameter;
class SerializableParameter extends Parameter {
}
exports.SerializableParameter = SerializableParameter;
class BodyParameter extends Parameter {
    getSchema() {
        return this.schema;
    }
    setSchema(schema) {
        this.schema = PropertyBuilder_1.toModel(schema);
    }
}
BodyParameter.TYPE = 'body';
exports.BodyParameter = BodyParameter;
class CookieParameter extends SerializableParameter {
}
CookieParameter.TYPE = 'cookie';
exports.CookieParameter = CookieParameter;
class FormParameter extends SerializableParameter {
    getDefaultCollectionFormat() {
        return 'multi';
    }
}
FormParameter.TYPE = 'formData';
exports.FormParameter = FormParameter;
class HeaderParameter extends SerializableParameter {
}
HeaderParameter.TYPE = 'header';
exports.HeaderParameter = HeaderParameter;
class PathParameter extends SerializableParameter {
    constructor() {
        super(...arguments);
        this.required = true;
    }
}
PathParameter.TYPE = 'path';
exports.PathParameter = PathParameter;
class QueryParameter extends SerializableParameter {
    getDefaultCollectionFormat() {
        return 'multi';
    }
}
QueryParameter.TYPE = 'query';
exports.QueryParameter = QueryParameter;
class RefParameter extends Parameter {
    constructor({ $ref } = {}) {
        super();
        this.set$ref($ref);
    }
    asDefault(ref) {
        this.set$ref(RefType_1.default.PARAMETER.getInternalPrefix() + ref);
        return this;
    }
    set$ref(ref) {
        this.genericRef = new GenericRef_1.default(RefType_1.default.PARAMETER, ref);
    }
    getRefFormat() {
        return this.genericRef.getFormat();
    }
    getSimpleRef() {
        return this.genericRef.getSimpleRef();
    }
}
RefParameter.TYPE = 'ref';
exports.RefParameter = RefParameter;
beanUtils_1.beanify(Parameter.prototype, [
    'name',
    'enum',
    'in',
    'description',
    'required',
    'type',
    'items',
    'collectionFormat',
    'default',
    'maximum',
    'exclusiveMaximum',
    'minimum',
    'exclusiveMinimum',
    'maxLength',
    'minLength',
    'pattern',
    'maxItems',
    'minItems',
    'uniqueItems',
    'multipleOf',
    'format',
]);
const TYPES = [
    BodyParameter,
    CookieParameter,
    FormParameter,
    HeaderParameter,
    PathParameter,
    QueryParameter,
    RefParameter,
];
function default_1(val) {
    if (val instanceof Parameter)
        return val;
    for (const ParameterType of TYPES) {
        if (ParameterType.TYPE == val.in) {
            const ret = beanUtils_1.apply(new ParameterType(val), val);
            return ret;
        }
    }
    if ('$ref' in val) {
        const rp = new RefParameter(val);
        return beanUtils_1.apply(rp, val);
    }
    throw new Error(`Could not resolve parameter type: ${val.in}`);
}
exports.default = default_1;
//# sourceMappingURL=parameters.js.map