"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const javaUtil_1 = require("../java/javaUtil");
const factory_1 = __importDefault(require("./factory"));
const beanUtils_1 = require("../java/beanUtils");
const GenericRef_1 = __importDefault(require("./refs/GenericRef"));
const RefType_1 = __importDefault(require("./refs/RefType"));
const Xml_1 = __importDefault(require("./Xml"));
class Property {
    constructor(obj, parent) {
        if (parent) {
            this.getParent = () => parent;
        }
    }
    getType() {
        return this.type || this.constructor.TYPE;
    }
    getFormat() {
        return this.format || this.constructor.FORMAT;
    }
    setXml(xml) {
        if (xml instanceof Xml_1.default) {
            this.xml = xml;
        }
        this.xml = new Xml_1.default(xml);
    }
    getVendorExtensions() {
        if (!this.vendorExtensions) {
            return javaUtil_1.newHashMap();
        }
        return this.vendorExtensions;
    }
    toString() {
        return `[${this.constructor.name}]${JSON.stringify(this, null, 2)}`;
    }
    toXml() { }
}
Property.allowedProps = [
    'title',
    'type',
    'format',
    'description',
    'name',
    'readOnly',
    'position',
    'access',
    'enum',
    'example',
    'externalDocs',
    'required',
    'xml',
    ['default', null, '_'],
];
exports.Property = Property;
class AbstractNumericProperty extends Property {
}
AbstractNumericProperty.allowedProps = [
    ...Property.allowedProps,
    'minimum',
    'maximum',
    'exclusiveMinimum',
    'exclusiveMaximum',
    'multipleOf',
];
exports.AbstractNumericProperty = AbstractNumericProperty;
function objectToPropertyMap(obj) {
    return javaUtil_1.newHashMap(...Object.keys(obj).map(key => [key, factory_1.default(obj[key], this)]));
}
class _ObjectProperty extends Property {
    constructor(obj, parent) {
        super(obj, parent);
        // this.setRequiredProperties(obj.required);
    }
    getAdditionalProperties() {
        return this._additionalProperties;
    }
    setAdditionalProperties(properties) {
        if (!properties) {
            this._additionalProperties = null;
        }
        if (properties.type) {
            this._additionalProperties = factory_1.default(properties);
        }
        else {
            this._additionalProperties = objectToPropertyMap(properties);
        }
    }
    setProperties(properties) {
        this.properties = objectToPropertyMap(properties);
    }
    getRequiredProperties() {
        const required = [];
        for (const [key, prop] of this.properties) {
            if (prop.required)
                required.push(key);
        }
        return required;
    }
    setRequiredProperties(required) {
        for (const [key, prop] of this.properties) {
            const isRequired = required ? required.indexOf(key) != -1 : false;
            if (beanUtils_1.has(prop, 'required') || isRequired) {
                prop.required = isRequired;
            }
        }
    }
}
_ObjectProperty.allowedProps = [
    ...Property.allowedProps,
    'additionalProperties',
    'required',
    'minProperties',
    'maxProperties',
    'anyOf',
    'allOf',
    'readOnly',
    'vendorExtensions',
    'xml',
];
class ObjectProperty extends _ObjectProperty {
}
ObjectProperty.TYPE = 'object';
ObjectProperty.allowedProps = [..._ObjectProperty.allowedProps, 'properties'];
exports.ObjectProperty = ObjectProperty;
class MapProperty extends _ObjectProperty {
}
MapProperty.TYPE = 'object';
exports.MapProperty = MapProperty;
class RefProperty extends Property {
    constructor(ref) {
        super();
        if (ref && ref.$ref)
            this.set$ref(ref.$ref);
        else if (typeof ref === 'string')
            this.set$ref(ref);
    }
    set$ref(ref) {
        this.genericRef = new GenericRef_1.default(RefType_1.default.DEFINITION, ref);
    }
    get$ref() {
        return this.genericRef.getRef();
    }
    asDefault(ref) {
        this.set$ref(RefType_1.default.DEFINITION.getInternalPrefix() + ref);
        return this;
    }
    getSimpleRef() {
        if (this.genericRef != null) {
            const simp = this.genericRef.getSimpleRef();
            return simp;
        }
    }
    getRefFormat() {
        if (this.genericRef != null) {
            return this.genericRef.getFormat();
        }
    }
}
RefProperty.TYPE = 'ref';
RefProperty.allowedProps = [...Property.allowedProps, '$ref'];
exports.RefProperty = RefProperty;
class ArrayProperty extends Property {
    items(items) {
        this.setItems(items);
        return this;
    }
    getItems() {
        return this._items;
    }
    setItems(items) {
        this._items = factory_1.default(items, this);
    }
}
ArrayProperty.TYPE = 'array';
ArrayProperty.allowedProps = [
    ...Property.allowedProps,
    'uniqueItems',
    'items',
    'maxItems',
    'minItems',
];
exports.ArrayProperty = ArrayProperty;
class BooleanProperty extends Property {
}
BooleanProperty.TYPE = 'boolean';
exports.BooleanProperty = BooleanProperty;
class StringProperty extends Property {
}
StringProperty.TYPE = 'string';
StringProperty.allowedProps = [
    ...Property.allowedProps,
    'minLength',
    'maxLength',
    'pattern',
];
exports.StringProperty = StringProperty;
class NumberProperty extends AbstractNumericProperty {
}
NumberProperty.TYPE = 'number';
exports.NumberProperty = NumberProperty;
class BaseIntegerProperty extends AbstractNumericProperty {
}
BaseIntegerProperty.TYPE = 'integer';
exports.BaseIntegerProperty = BaseIntegerProperty;
class IntegerProperty extends BaseIntegerProperty {
}
IntegerProperty.FORMAT = 'integer';
exports.IntegerProperty = IntegerProperty;
class LongProperty extends NumberProperty {
}
LongProperty.FORMAT = 'long';
exports.LongProperty = LongProperty;
class Int64Property extends LongProperty {
}
Int64Property.FORMAT = 'int64';
Int64Property.TYPE = 'integer';
exports.Int64Property = Int64Property;
class Int32Property extends BaseIntegerProperty {
}
Int32Property.FORMAT = 'int32';
Int32Property.TYPE = 'integer';
exports.Int32Property = Int32Property;
class DoubleProperty extends NumberProperty {
}
DoubleProperty.FORMAT = 'double';
exports.DoubleProperty = DoubleProperty;
class DecimalProperty extends NumberProperty {
}
DecimalProperty.FORMAT = 'decimal';
exports.DecimalProperty = DecimalProperty;
class FloatProperty extends NumberProperty {
}
FloatProperty.FORMAT = 'float';
exports.FloatProperty = FloatProperty;
class ByteArrayProperty extends StringProperty {
}
ByteArrayProperty.FORMAT = 'byte-array';
exports.ByteArrayProperty = ByteArrayProperty;
class ByteProperty extends StringProperty {
}
ByteProperty.FORMAT = 'byte';
exports.ByteProperty = ByteProperty;
class BinaryProperty extends StringProperty {
}
BinaryProperty.FORMAT = 'binary';
exports.BinaryProperty = BinaryProperty;
class EmailProperty extends StringProperty {
}
EmailProperty.FORMAT = 'email';
exports.EmailProperty = EmailProperty;
class DateProperty extends StringProperty {
}
DateProperty.FORMAT = 'date';
exports.DateProperty = DateProperty;
class DateTimeProperty extends StringProperty {
}
DateTimeProperty.FORMAT = 'date-time';
exports.DateTimeProperty = DateTimeProperty;
class UriProperty extends StringProperty {
}
UriProperty.FORMAT = 'uri';
exports.UriProperty = UriProperty;
class UrlProperty extends UriProperty {
}
UrlProperty.FORMAT = 'url';
exports.UrlProperty = UrlProperty;
class UUIDProperty extends Property {
}
UUIDProperty.FORMAT = 'uuid';
UUIDProperty.TYPE = 'string';
exports.UUIDProperty = UUIDProperty;
class NullProperty extends Property {
}
NullProperty.TYPE = 'null';
exports.NullProperty = NullProperty;
class FileProperty extends StringProperty {
}
FileProperty.TYPE = 'file';
exports.FileProperty = FileProperty;
//# sourceMappingURL=properties.js.map